import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Point;


/**
 * This class implements a logo-style turtle that can move and draw lines.
 *
 * @author malcolmr
 * @version 16 July 2012
 */
public class Turtle {

    private Window myWindow;
    private Point myPosition;
    private int myRotation;   
    private boolean myPenDown;
    private Color myPenColor;
    private BasicStroke myStroke;
    private int myPenWidth;
    
    /**
     * Create a new turtle in its own window.
     * Initially the turtle is in the middle of the window.
     */
    public Turtle() {
        this(new Window());
    }
    
    /**
     * Create a new turtle in the specified window.
     * Initially the turtle is in the middle of the window.
     */
    public Turtle(Window w) {
        myWindow = w;
        w.addTurtle(this);
        
        myPosition = new Point(0,0);
        myRotation = 0; 
        myPenDown = true;
        myPenColor = Color.BLACK;        
        myPenWidth = 1;
        myStroke = new BasicStroke(myPenWidth);
    }
    
    /**
     * Read the turtle's current position.
     */
    public Point position() {
        return myPosition;        
    }

    /**
     * Read the turtle's current rotation.
     */
    public int rotation() {
        return myRotation;
    }
    
    /**
     * Rotate the turtle anticlockwise by the specified angle (in degrees)
     */
    public void rotate(int angle) {
        myRotation += angle;
        myRotation = myRotation % 360;
        // notify the window that the turtle has moved
        myWindow.stateChanged();  
    }

    /**
     * Test if the pen is down
     */
    public boolean isPenDown() {
        return myPenDown;
    }
    
    /**
     * Raise or lower the pen.
     */
    public void setPenDown(boolean down) {
        myPenDown = down;
    }
    
    /**
     * Get the current pen width
     */
    public int penWidth() {
        return myPenWidth;
    }
    
    /**
     * Set the current pen width to the specified value.
     */
    public void setPenWidth(int width) {
        myPenWidth = width;
        myStroke = new BasicStroke(myPenWidth, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    }

    /**
     * Get the current pen colour
     */
    public Color penColor() {
        return myPenColor;
    }
    
    /**
     * Set the current pen colour to the specified value.
     */
    public void setPenColor(Color color) {
        myPenColor = color;
    }

    /**
     * Move the turtle forwards the specified distance.
     * This will draw a line if the pen is down.
     */
    public void move(int distance) {
        // remember the old position
        int oldX = myPosition.x;
        int oldY = myPosition.y;
        
        // calculate the new position
        myPosition.x = (int)Math.round(oldX + Math.cos(Math.toRadians(myRotation)) * distance);
        myPosition.y = (int)Math.round(oldY + Math.sin(Math.toRadians(myRotation)) * distance);
        
        // if the pen is down draw a line joining old and new positions
        if (myPenDown) {
            myWindow.drawLine(oldX, oldY, myPosition.x, myPosition.y, myPenColor, myStroke);
        }
        // tell the window the image has changed so it can repaint.
        myWindow.stateChanged();
    }
    
}
